%macro BRR_FREQ_PV(	INFILE=,
				REPLI_ROOT=,
				BYVAR=,
				PV_ROOT=,
				OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT		= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR			= 	BREAKDOWN VARIABLES
PV_ROOT		= 	ROOT OF THE 5 PROFICIENCY LEVEL VARIABLES NAMES
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0;
RUN;


%DO I = 1 %TO 5;

	PROC SORT DATA=&INFILE
		OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &PV_ROOT&I);
		BY &BYVAR &PV_ROOT&I;
	RUN;

	%DO J = 0 %TO 80;

		PROC FREQ DATA=BRRDATA NOPRINT;
			TABLE &PV_ROOT&I /OUT=FREQ_TEMP ;
			BY &BYVAR;
			WEIGHT &REPLI_ROOT&J;
		RUN;

		DATA FREQ_TEMP;
			SET FREQ_TEMP;
			K=&I;
			L=&J;
			&PV_ROOT=&PV_ROOT&I;
			KEEP &BYVAR K L PERCENT &PV_ROOT;
		RUN;

		PROC APPEND BASE = BRR_TEMP0 DATA=FREQ_TEMP;
		RUN;

	%END;

%END;

PROC SORT DATA=BRR_TEMP0;
	BY &BYVAR L &PV_ROOT;
RUN;

PROC TRANSPOSE DATA=BRR_TEMP0 OUT=BRR_TEMP1 PREFIX=PV;
	BY &BYVAR L &PV_ROOT;
	VAR PERCENT;
RUN; 

DATA BRR_TEMP1;
	SET BRR_TEMP1;
	DROP _NAME_ _LABEL_;
RUN;

DATA BRR_TEMP2(DROP=STAT FIN1-FIN5 MESVAR)BRR_TEMP3(KEEP=&BYVAR &PV_ROOT STAT FIN1-FIN5 MESVAR);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =(PV1+PV2+PV3+PV4+PV5)/5;
		FIN1=PV1;
		FIN2=PV2;
		FIN3=PV3;
		FIN4=PV4;
		FIN5=PV5;
		MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR &PV_ROOT;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR &PV_ROOT;
RUN;
DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR &PV_ROOT;
	ARRAY A (5)
		PV1-PV5;
	ARRAY B (5)
		FIN1-FIN5;
	ARRAY C (5)
		VAR1-VAR5;
	DO I=1 TO 5;
		C(I)=(1/20)*((A(I)-B(I))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VAR1 VAR2 VAR3 VAR4 VAR5;
	BY &BYVAR &PV_ROOT;
	OUTPUT OUT=BRR_TEMP5 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

OPTIONS NOTES;

DATA &OUTFILE;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR &PV_ROOT;
	SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
	FINVAR=(SAMP+(1.2*MESVAR));
	SESTAT=(FINVAR)**0.5;
	KEEP &BYVAR &PV_ROOT STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP BRRDATA;
RUN;

%MEND BRR_FREQ_PV;


