%MACRO BRR_PROCMEAN_DIF_PV(INFILE =,
                           REPLI_ROOT =,
                           BYVAR =,
                           PV_ROOT =,
                           COMPARE =,
	                       CATEGORY =,
                           STAT =,
                           OUTFILE =);

/*

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
PV_ROOT		= 	ROOT OF THE 5 PLAUSIBLE VALUES VARIABLES NAMES
COMPARE		=	BREAKDOWN VARIABLE NAME FOR WICH CATEGORY CONTRASTS ARE REQUESTED
CATEGORY	=	LIST OF THE "COMPARE" VARIABLE CATEGORIES FOR WHICH A CONTRAT IS REQUESTED
STAT		= 	REQUESTED STATISTIQUE
				SUMWGT		=	SUM OF THE WEIGHT
				MEAN		= 	MEAN
				VAR			=	VARIANCE
				STD			=	STANDARD DEVIATION
				CV			=	COEFFICIENT OF VARIATION
				MEDIAN		=	MEDIAN
				Q1			=	FIRST QUARTILE
				Q3			=	THIRD QUARTILE
				QRANGE		=	RANGE BETWEEN Q1 AND Q3
				PX			=	PERCENTILE, WITH X BETWEEN 1 AND 99
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 ;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &COMPARE &PV_ROOT.1-&PV_ROOT.5);
	BY &BYVAR &COMPARE;
RUN;

%DO I = 0 %TO 80;

	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &PV_ROOT.1 - &PV_ROOT.5 ;
		BY &BYVAR &COMPARE;
		WEIGHT &REPLI_ROOT&I;
		OUTPUT OUT=MEAN_TEMP &STAT=PV1 - PV5;
 	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&I;
 	RUN;

	PROC APPEND BASE = BRR_TEMP0 DATA=MEAN_TEMP;
	RUN;

%END;


%LET DEBUT=1;
%LET SUIVANT=2;
%LET COMPTE=1;
%LET I=1;

%DO %WHILE(%LENGTH(%SCAN(&CATEGORY,&I)));
	%LET I=%EVAL(&I+1);
%END;

%LET NBCAT=%EVAL(&I-1);
%LET NBDIF=%EVAL((&NBCAT*(&NBCAT-1))/2);
%LET ORDRE=1;

%DO J=1 %TO &NBDIF;
	%DO K=&DEBUT %TO &NBCAT-1;
		%LET CAT1=%SCAN(&CATEGORY,&DEBUT);
		%LET CAT2=%SCAN(&CATEGORY,&SUIVANT);

		DATA BRR_DIF1;
			SET BRR_TEMP0 (RENAME=(PV1=M1PV1 PV2=M1PV2 PV3=M1PV3 PV4=M1PV4 PV5=M1PV5));
			LENGTH CONTRAST $5;
			CONTRAST="&CAT1.-&CAT2";
			IF (&COMPARE=&CAT1);
			KEEP &BYVAR L CONTRAST M1PV1-M1PV5 &compare;
		RUN;

		DATA BRR_DIF2 ;
			SET BRR_TEMP0 (RENAME=(PV1=M2PV1 PV2=M2PV2 PV3=M2PV3 PV4=M2PV4 PV5=M2PV5));
			IF (&COMPARE=&CAT2);
			KEEP &BYVAR L M2PV1-M2PV5 &compare;
		RUN;



PROC SORT DATA=BRR_DIF1;
			BY &BYVAR L;
		RUN;

		PROC SORT DATA=BRR_DIF2;
			BY &BYVAR L;
		RUN;

		DATA BRR_TEMP_DIF;
			MERGE BRR_DIF1 BRR_DIF2;
			BY &BYVAR L;
			IF (M1PV1 EQ . OR M2PV1 EQ .) THEN DELETE;
			PV1=M1PV1-M2PV1;
			PV2=M1PV2-M2PV2;
			PV3=M1PV3-M2PV3;
			PV4=M1PV4-M2PV4;
			PV5=M1PV5-M2PV5;
			ORDRE=&ORDRE;
			KEEP &BYVAR CONTRAST L PV1-PV5 ORDRE;
		RUN;

 		PROC APPEND BASE = BRR_TEMP1 DATA=BRR_TEMP_DIF;
		RUN;

		%LET SUIVANT=%EVAL(&SUIVANT+1);
		%LET COMPTE=%EVAL(&COMPTE+1);
		%LET ORDRE=%EVAL(&ORDRE+1);

	%END;

%LET DEBUT=%EVAL(&DEBUT+1);
%LET SUIVANT=%EVAL(&DEBUT+1);

%END;



DATA BRR_TEMP2(DROP=STAT FIN1-FIN5 MESVAR) BRR_TEMP3(KEEP=&BYVAR CONTRAST ORDRE STAT FIN1-FIN5 MESVAR);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =(PV1+PV2+PV3+PV4+PV5)/5;
		FIN1=PV1;
		FIN2=PV2;
		FIN3=PV3;
		FIN4=PV4;
		FIN5=PV5;
		MESVAR=(((STAT-FIN1)**2)+((STAT-FIN2)**2)+((STAT-FIN3)**2)+((STAT-FIN4)**2)+((STAT-FIN5)**2))/4;
		OUTPUT BRR_TEMP3;
   END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR CONTRAST;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR CONTRAST;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR CONTRAST;
	ARRAY A (5)
		PV1-PV5;
	ARRAY B (5)
		FIN1-FIN5;
	ARRAY C(5)
		VAR1-VAR5;
	DO I=1 TO 5;
		C(I)=(1/20)*((A(I)-B(I))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VAR1 VAR2 VAR3 VAR4 VAR5;
	BY &BYVAR CONTRAST;
	OUTPUT OUT=BRR_TEMP5 SUM=SS1 SS2 SS3 SS4 SS5;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR CONTRAST;
	SAMP=(SS1+SS2+SS3+SS4+SS5)/5;
	FINVAR=(SAMP+(1.2*MESVAR));
	SESTAT=(FINVAR)**0.5;
	KEEP &BYVAR CONTRAST ORDRE STAT SESTAT;
RUN;

OPTIONS NOTES;

PROC SORT DATA=BRR_TEMP6 OUT=&OUTFILE (DROP=ORDRE);
	BY &BYVAR ORDRE;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6
	MEAN_TEMP BRRDATA BRR_DIF1 BRR_DIF2 BRR_TEMP_DIF;
RUN;

%MEND BRR_PROCMEAN_DIF_PV;

