%MACRO BRR_PROCMEAN(INFILE =,
                    REPLI_ROOT =,
                    BYVAR = ,
                    VAR =,
                    STAT =,
                    OUTFILE =);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE	= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATE weight VARIABLE NAMES. 
			FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
VAR			= 	VARIABLE ON WHICH THE REQUESTED STATISTIC WILL BE COMPUTED
STAT		= 	REQUESTED STATISTIC
				SUMWGT	=	SUM OF THE WEIGHTS
				MEAN		= 	MEAN
				VAR		=	VARIANCE
				STD		=	STANDARD DEVIATION
				CV		=	COEFFICIENT OF VARIATION
				MEDIAN	=	MEDIAN
				Q1		=	FIRST QUARTILE
				Q3		=	THIRD QUARTILE
				QRANGE	=	RANGE BETWEEN Q1 AND Q3
				PX		=	PERCENTILE, WITH X BETWEEN 1 AND 99
OUTFILE	= 	FILE WITH THE ESTIMATES AND THEIR STANDARD ERROR ESTIMATES

*/

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA (KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &VAR);
BY &BYVAR;
RUN;

%DO I = 0 %TO 80;
 	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &VAR ;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT&I;
		OUTPUT OUT=MEAN_TEMP &STAT=PV;
	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&I;
	RUN;

	PROC APPEND BASE = BRR_TEMP1 DATA=MEAN_TEMP;
 	RUN;
%END;

DATA BRR_TEMP2(KEEP=&BYVAR PV)BRR_TEMP3(KEEP=&BYVAR STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =PV;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR;
	VARI=((PV-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &BYVAR;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;

OPTIONS NOTES;
	DATA &OUTFILE;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR;
	SESTAT=(SS)**0.5;
	KEEP &BYVAR STAT SESTAT;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 MEAN_TEMP BRRDATA;
RUN;

%MEND BRR_PROCMEAN;

